@@ -12,9 +12,9 @@ setGeneric("dti.smooth", function(object, ...) standardGeneric("dti.smooth"))
 
 setMethod("dti.smooth", "dtiData", function(object,hmax=5,hinit=NULL,lambda=20,
                                             rho=1,graph=FALSE,slice=NULL,quant=.8,
-                                            minanindex=NULL,hsig=2.5,lseq=NULL, method="nonlinear",varmethod="residuals",rician=TRUE,niter=5,varmodel="local",volseq=TRUE) {
+                                            minanindex=NULL,hsig=2.5,lseq=NULL, method="nonlinear",varmethod="residuals",rician=TRUE,niter=5,varmodel="local") {
 switch(method,"linear" = dtilin.smooth(object,hmax,hinit,lambda,rho,graph,slice,quant,minanindex,hsig,lseq,varmethod,varmodel),
-              "nonlinear" =  dtireg.smooth(object,hmax,hinit,lambda,rho,graph,slice,quant,minanindex,hsig,lseq,varmethod,rician,niter,varmodel,volseq=volseq))
+              "nonlinear" =  dtireg.smooth(object,hmax,hinit,lambda,rho,graph,slice,quant,minanindex,hsig,lseq,varmethod,rician,niter,varmodel))
 }
 )
 dtilin.smooth <- function(object,hmax=5,hinit=NULL,lambda=52,
@@ -30,14 +30,30 @@ dtilin.smooth <- function(object,hmax=5,hinit=NULL,lambda=52,
     if (!adimpro) cat("No graphical output! Install package adimpro from CRAN!\n")
     graph <- graph & adimpro
   }
-  args <- match.call()
+  args <- sys.call(-3)
+  args <- c(object@call,args)
   s0ind <- object@s0ind
-  s0 <- object@si[,,,s0ind]
-  if(length(s0ind)>1) s0 <- apply(s0,1:3,mean) 
-  si <- object@si[,,,-s0ind]
-  ngrad <- object@ngrad-length(s0ind)
+  ns0 <- length(s0ind)
+  ngrad <- object@ngrad
   ddim0 <- object@ddim0
   ddim <- object@ddim
+  z <- .Fortran("outlier",
+                as.integer(object@si),
+                as.integer(prod(ddim)),
+                as.integer(ngrad),
+                as.logical((1:ngrad)%in%s0ind),
+                as.integer(ns0),
+                si=integer(prod(ddim)*ngrad),
+                index=integer(prod(ddim)),
+                lindex=integer(1),
+                DUPL=FALSE,
+                PACKAGE="dti")[c("si","index","lindex")]
+  si <- array(z$si,c(ddim,ngrad))
+  object@si <- si
+  index <- if(z$lindex>0) z$index[1:z$lindex] else numeric(0)
+  s0 <- si[,,,s0ind]
+  if(length(s0ind)>1) s0 <- apply(s0,1:3,mean) 
+  si <- si[,,,-s0ind]
   xind <- object@xind
   yind <- object@yind
   zind <- object@zind
@@ -48,8 +64,8 @@ dtilin.smooth <- function(object,hmax=5,hinit=NULL,lambda=52,
   Bcov <- btb%*%t(btb)
   btbsvd <- svd(btb)
   solvebtb <- btbsvd$u %*% diag(1/btbsvd$d) %*% t(btbsvd$v)
-  
   dtobject <- dtiTensor(object,method="linear",varmethod=varmethod)
+  scale <- dtobject@scale
   mask <- dtobject@mask
   y <- dtobject@D
   sigma2 <- dtobject@sigma
@@ -142,6 +158,7 @@ dtilin.smooth <- function(object,hmax=5,hinit=NULL,lambda=52,
      show.image(make.image(65535*img/max(img)),xaxt="n",yaxt="n")
      title(paste("Dzz: mean",signif(mean(z$D[6,,,][mask]),3),"max",signif(max(z$D[6,,,][mask]),3)))
   }
+  ngrad <- ngrad-length(s0ind)
   hincr <- 1.25^(1/3)
   if(is.null(hinit)){
   hakt0 <- 1
@@ -256,9 +273,26 @@ dtilin.smooth <- function(object,hmax=5,hinit=NULL,lambda=52,
     k <- k+1
     lambda0 <- lambda*lseq[k]*scorrfactor     
   }
-
+#  replace non-tensors (with negative eigenvalues) by a small isotropic tensor 
+      ind <- array(.Fortran("dti3Dev",
+                           as.double(z$D),
+                           as.integer(n1),
+                           as.integer(n2),
+                           as.integer(n3),
+                           as.logical(mask),
+                           ev=double(3*n1*n2*n3),
+                           DUPL=FALSE,
+                           PACKAGE="dti")$ev,c(3,n1,n2,n3))[1,,,]<1e-6
+       if(sum(ind&mask)>0){
+           dim(z$D) <- c(6,n1*n2*n3)
+           z$D[c(1,4,6),ind&mask] <- 1e-6
+           z$D[c(2,3,5),ind&mask] <- 0
+           dim(z$D) <- c(6,n1,n2,n3)
+       }
+  
   invisible(new("dtiTensor",
                 list( hmax = hmax),
+                call = args,
                 D     = z$D,
                 th0   = th0,
                 btb   = btb,
@@ -276,210 +310,10 @@ dtilin.smooth <- function(object,hmax=5,hinit=NULL,lambda=52,
                 zind  = zind,
                 voxelext = voxelext,
                 source= source,
+                outlier = index,
+                scale = scale,
                 method= "linear")
             )
 
 }
 
-setMethod("dti.smooth", "dtiTensor", function(object,hmax=5,lambda=20,rho=1,graph=FALSE,
-                                            slice=NULL,quant=.8,minanindex=NULL){
-#
-#
-cat("Functionality for smoothing the diffusion tensor is only included for demonstartion purposes.\n
-We do not advise to smooth the tensor, neither using a Rimannian or log-Euclidian or Euclidian metric.\n
-Please use function dti.smooth on a dtiData-object ... .\n")
-    rdtianiso(object,hmax=hmax,lambda=lambda,rho=rho,graph=graph,
-              slice=slice,quant=quant,minanindex=minanindex)
-  }
-)
-
-
-
-rdtianiso <- function(dtobject,hmax=5,lambda=20,rho=1,graph=FALSE,slice=NULL,quant=.8,minanindex=NULL){
-  cat("Smoothing DT with Riemann Tensor Metric\n")
-  if (graph) {
-    adimpro <- require(adimpro)
-    if (!adimpro) cat("No graphical output! Install package adimpro from CRAN!\n")
-    graph <- graph & adimpro
-  }
-  args <- match.call()
-  btb<-dtobject@btb
-  Bcov <- btb%*%t(btb)
-  y <- dtobject@D
-  sigma2 <- dtobject@sigma
-  scorr <- dtobject@scorr
-  h0 <- dtobject@bw
-  mask <- dtobject@mask
-  method <- dtobject@method
-  th0 <- dtobject@th0
-  ddim <- dtobject@ddim
-  ddim0 <- dtobject@ddim0
-  xind <- dtobject@xind
-  yind <- dtobject@yind
-  zind <- dtobject@zind
-  ngrad <- dtobject@ngrad
-  voxelext <- dtobject@voxelext
-  if(is.null(voxelext)) vext <- c(1,1,1) else vext <- voxelext/min(voxelext)
-  source <- dtobject@source
-  rm(dtobject)
-  gc()
-  dimy <- dim(y)
-  if(length(dimy)!=4||dimy[1]!=6) stop("y does not contain 3D diffusion tensor image")
-  n1<-dimy[2]
-  n2<-dimy[3]
-  n3<-dimy[4]
-  n<-n1*n2*n3
-  sigma2[sigma2<=mean(sigma2)*1e-5]<- mean(sigma2)*1e-5
-  z <- .Fortran("projdt",
-                as.double(y),
-                as.integer(n1),
-                as.integer(n2),
-                as.integer(n3),
-                D=double(6*n),
-                anindex=double(n),
-                andirection=double(3*n),
-                det=double(n),
-                mask=as.logical(mask),
-                DUP=FALSE,
-                PACKAGE="dti")[c("D","anindex","andirection","det","mask")]
-  y <- array(z$D,dimy)
-  z$bi <- 1/sigma2
-  dim(z$D) <- dimy
-  dim(z$anindex) <-dim(z$det) <-dim(z$mask) <- dimy[-1]
-  z$mask <- array(z$mask,dimy[-1])&mask
-  dim(z$andirection) <- c(3,dimy[-1]) 
-#
-#  initial state for h=1
-#
-  if(graph){
-     oldpar <- par(mfrow=c(3,3),mar=c(1,1,3,.25),mgp=c(2,1,0))
-     on.exit(par(oldpar))
-     if(is.null(slice)) slice<-n3%/%2
-     class(z) <- "dti"
-     img<-z$D[1,,,slice]
-     show.image(make.image(65535*img/max(img)),xaxt="n",yaxt="n")
-     title(paste("Dxx: min",signif(min(img),3),"max",signif(max(img),3)))
-     img<-z$D[2,,,slice]
-     show.image(make.image(img),xaxt="n",yaxt="n")
-     title(paste("Dxy: min",signif(min(img),3),"max",signif(max(img),3)))
-     img<-z$D[3,,,slice]
-     show.image(make.image(img),xaxt="n",yaxt="n")
-     title(paste("Dxz: min",signif(min(img),3),"max",signif(max(img),3)))
-     show.image(make.image(z$anindex[,,slice]),xaxt="n",yaxt="n")
-     title(paste("Anisotropy index  range:",signif(min(z$anindex[z$mask]),3),"-",
-                  signif(max(z$anindex[z$mask]),3)))
-     img<-z$D[4,,,slice]
-     show.image(make.image(65535*img/max(img)),xaxt="n",yaxt="n")
-     title(paste("Dyy: min",signif(min(img),3),"max",signif(max(img),3)))
-     img<-z$D[5,,,slice]
-     show.image(make.image(img),xaxt="n",yaxt="n")
-     title(paste("Dyz: min",signif(min(img),3),"max",signif(max(img),3)))
-     andir2.image(z,slice,quant=quant,minanindex=minanindex,xaxt="n",yaxt="n")
-     title(paste("Directions (h=1), slice",slice))
-     ni<-z$bi[,,slice]*sigma2[,,slice]
-     show.image(make.image(65535*ni/max(ni)),xaxt="n",yaxt="n")
-     title(paste("sum of weights  mean=",signif(mean(z$bi[z$mask]*sigma2[z$mask]),3)))
-     img<-z$D[6,,,slice]
-     show.image(make.image(65535*img/max(img)),xaxt="n",yaxt="n")
-     title(paste("Dzz: min",signif(min(img),3),"max",signif(max(img),3)))
-  }
-  hincr <- 1.25^(1/3)
-  hakt0 <- 1
-  hakt <- hincr
-  lambda0 <- lambda
-  while( hakt <= hmax) {
-    if (any(h0 >= 0.25)) {
-       corrfactor <- Spatialvar.gauss(hakt0/0.42445/4,h0,3) /
-        Spatialvar.gauss(h0,1e-5,3) /
-        Spatialvar.gauss(hakt0/0.42445/4,1e-5,3)
-        lambda0 <- lambda * corrfactor
-     cat("Correction factor for spatial correlation",signif(corrfactor,3),"\n")
-}
-     usize<-(2*ceiling(hakt+1))^3
-     mask[z$anindex<0.1]<-FALSE
-     z <- .Fortran("rawsdti",
-                as.double(y),
-                as.double(z$D),
-                bi=as.double(z$bi),
-                anindex=as.double(z$anindex),
-                andirection=as.double(z$andirection),
-                det=as.double(z$det),
-                as.double(Bcov),
-                as.double(sigma2),
-                as.integer(n1),
-                as.integer(n2),
-                as.integer(n3),
-                as.double(hakt),
-                as.double(vext),
-                as.double(rho),
-                as.double(lambda0),
-                D=double(6*n),
-                mask=as.logical(z$mask),
-                double(usize),# array for weights
-                double(9*usize),# array for complete diffusion tensors with pos. weights
-                as.integer(usize),# upper bound for size of U(x)
-                double(9*usize),# array for Remannian Log Maps 
-                DUP=FALSE,
-                PACKAGE="dti")[c("D","bi","anindex","andirection","det","mask")]
-     dim(z$bi) <- dim(z$anindex) <- dim(z$det) <- dim(z$mask) <- dimy[-1]
-     dim(z$D) <- dimy
-     dim(z$andirection) <- c(3,dimy[-1]) 
-     if(graph){
-     
-     class(z) <- "dti"
-     img<-z$D[1,,,slice]
-     show.image(make.image(65535*img/max(img)),xaxt="n",yaxt="n")
-     title(paste("Dxx: min",signif(min(img),3),"max",signif(max(img),3)))
-     img<-z$D[2,,,slice]
-     show.image(make.image(img),xaxt="n",yaxt="n")
-     title(paste("Dxy: min",signif(min(img),3),"max",signif(max(img),3)))
-     img<-z$D[3,,,slice]
-     show.image(make.image(img),xaxt="n",yaxt="n")
-     title(paste("Dxz: min",signif(min(img),3),"max",signif(max(img),3)))
-     show.image(make.image(z$anindex[,,slice]),xaxt="n",yaxt="n")
-     title(paste("Anisotropy index  range:",signif(min(z$anindex[z$mask]),3),"-",
-                  signif(max(z$anindex[z$mask]),3)))
-     img<-z$D[4,,,slice]
-     show.image(make.image(65535*img/max(img)),xaxt="n",yaxt="n")
-     title(paste("Dyy: min",signif(min(img),3),"max",signif(max(img),3)))
-     img<-z$D[5,,,slice]
-     show.image(make.image(img),xaxt="n",yaxt="n")
-     title(paste("Dyz: min",signif(min(img),3),"max",signif(max(img),3)))
-     andir2.image(z,slice,quant=quant,minanindex=minanindex,xaxt="n",yaxt="n")
-     title(paste("Directions (h=",signif(hakt,3),"), slice",slice))
-     ni<-z$bi[,,slice]*sigma2[,,slice]
-     show.image(make.image(65535*ni/max(ni)),xaxt="n",yaxt="n")
-     title(paste("sum of weights  mean=",signif(mean(z$bi[z$mask]*sigma2[z$mask]),3)))
-     img<-z$D[6,,,slice]
-     show.image(make.image(65535*img/max(img)),xaxt="n",yaxt="n")
-     title(paste("Dyy: min",signif(min(img),3),"max",signif(max(img),3)))
-     }
-     cat("h=",signif(hakt,3),"Quantiles (.5, .75, .9, .95, 1) of anisotropy index",signif(quantile(z$anindex[z$mask],c(.5, .75, .9, .95, 1)),3),"\n")
-     hakt0<-hakt
-     hakt <- hakt*hincr
-  }
-#z <- list(D=z$D,bi=z$bi,anindex=z$anindex,andirection=z$andirection,mask=z$mask, ddim0=ddim0,xind=xind,yind=yind,zind=zind,InvCov=Bcov,call=args)
-#class(z) <- "dti"
-
-  invisible(new("dtiTensor",
-                list(metrik="Riemann"),
-                method = method,
-                D     = z$D,
-                th0   = th0,
-                sigma = z$sigma2hat,
-                scorr = scorr,
-                bw    = h0,
-                mask  = mask,
-                hmax  = hmax,
-                btb   = btb,
-                ngrad = ngrad, # = dim(btb)[2]
-                ddim  = as.integer(ddim),
-                ddim0 = as.integer(ddim0),
-                xind  = xind,
-                yind  = yind,
-                zind  = zind,
-                voxelext = voxelext,
-                source= source)
-            )
-}
-
